/********************************
 *
 * NAME:
 * ----
 * synchronize_react/main.c
 *
 * AUTHORS:
 * -------
 * Alvaro Gutierrez -- aguti@etsit.upm.es
 * Alexandre Campo  -- acampo@ulb.ac.be
 *
 * REVISION HISTORY:
 *-----------------
 * 12/12/2008
 *
 * ADDITIONAL NOTES:
 * ----------------
 * A test for the e-randb board.
 * The e-puck reads and send data to the E-RANDB.
 * The data means the orientation a robot has with 
 * respect to another, and they try to move towards
 * that direction and therefore to align.
 */


#include <p30f6014a.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <btcom.h>
#include <e_randb.h>

#include <e_agenda.h>
#include <e_motors.h>

#include <e_uart_char.h>
#include <e_init_port.h>

#define EPSILON_ANGLE 5

void turnToAngle ( int angle);
void Stop ( void );
void TurnRight ( void );
void TurnLeft ( void );
int turnSpeed;
int main() {

	/*system initialization */
	e_init_port();
	/* Init UART1 for bluetooth */
	e_init_uart1();
	/* Init E-RANDB board */
	e_init_randb();

	/* Init Agendas */
	e_start_agendas_processing();
	/* Init motors */
	e_init_motors();

	/* Wait for a command comming from bluetooth IRCOMTEST on pc directory*/
	btcomWaitForCommand('s');
	
	/* Range is tunable by software. 
	 * 0 -> Full Range (1m. approx depending on light conditions )
	 * 255 --> No Range (0cm. approx, depending on light conditions */
	e_randb_set_range(150);

	/* At some point we tought that the board could just take
	 * data en leave the calculations for the robot.
	 * At the moment, it is better to allow the board to do the calculations */
	e_randb_set_calculation(ON_BOARD);

	/* Store light conditions to use them as offset for the calculation 
	 * of the range and bearing */
	e_randb_store_light_conditions();

	/* A variable to hold a delay */
	
	/* Print on the bluetooth */
	char tmp2[50];

	unsigned char data = 0;

	float angle=0;
	int angleDeg;
	char max_peak = -1;
	int oldError = 0;
	
	e_randb_set_range(160);
	e_randb_set_calculation(ON_BOARD);
	e_randb_store_light_conditions();

	long int delay;
	int counter = 0;
	while(1){
		
		/* If something received */
		if(e_randb_get_if_received()!=0){
		
			/* Get data of reception */
			int data = e_randb_get_data();
			
			/* Get Max Sensor */
			max_peak = e_randb_get_sensor();
			
			/* Get angle of reception */
			angle = e_randb_get_bearing();
			angleDeg = (int) (angle * 180.0 / 3.14159);
			if ( angleDeg < 0 ) angleDeg += 360;

			/* Calc error */
			int error = angleDeg - data;
			/* Move to the commented angle */
			turnToAngle( error );

			/* DEBUG */
			sprintf(tmp2,"%d %d %d %d %2f", angleDeg, data, error, max_peak, angle);
			btcomSendString(tmp2);
			btcomSendString("\n");
			/* DEBUG */

			/* Waiting Time */
			for ( delay = 0 ; delay < 4000; delay++){
				__asm__("nop");
			}
			counter = 0;
		}
		/* If nothing receive, wait a bit of time to
		 * not colapse the E-RANDB.
		 * Will be solved once the UART2 is working*/
		else
		{	
			for ( delay = 0 ; delay < 4000; delay++){
				__asm__("nop");
			}
			counter++;
		}

		/* If first reception or not received during 20 cycles */
		if(max_peak == -1 || counter >= 20)
		{
			/* Send to all the sources */
			e_randb_send_all_data(angleDeg);
			/* Reset counter of nothing received */
			counter = 0;
		}
		/* If not, sent to the 2 sensors in between the maximum peak received */
		else
		{
			/* If the sensors was the last one */
			if(max_peak == 11)
			{
				/* Prepare data to be send on the first and last */
				e_randb_store_data(0,angleDeg);
				e_randb_store_data(11,angleDeg);
			}
			/* If it is not the las one */
			else
			{
				/* Prepare data to the 2 sensors in between */
				e_randb_store_data(max_peak,angleDeg);
				e_randb_store_data((max_peak+1),angleDeg);
			}
			/* Send all the data simultaneouslly */
			e_randb_send_data();
		}

		/* Waiting Time */
		for ( delay = 0 ; delay < 4000; delay++){
			__asm__("nop");
		}
	}
	return 0;
}

/* Turn to the received angle */
void turnToAngle ( int angle ){


	angle += 180;
	if ( angle > 180 ) angle -= 360;
	if ( angle < -180 ) angle += 360;

	int error = fabs(angle);
	
	if(error < 5) turnSpeed = 0;
	else if(error < 40) turnSpeed = 100;
	else if(error < 60) turnSpeed = 100;
	else if(error < 80) turnSpeed = 150;
	else if(error < 120) turnSpeed = 200;
	else if(error < 170) turnSpeed = 300;
	else turnSpeed=500;

	/* DEBUG */
	/*char tmp2[100];*/
	/*sprintf(tmp2,"%d ", angle);*/
	/*btcomSendString(tmp2);*/
	/*btcomSendString("\n");*/
	/* DEBUG */

	/* If angle allright , STOP */
	if ( fabs(angle) < EPSILON_ANGLE ){
		Stop();
	}
	/* If angle not allright , MOVE*/
	else{
		if ( angle > 0 ) TurnLeft();
		else TurnRight();
	}
}

void Stop ( void ){
	e_set_speed_left( 0 );
	e_set_speed_right( 0 );
}

void TurnRight ( void ){
	e_set_speed_left( turnSpeed );
	e_set_speed_right( -turnSpeed );
}

void TurnLeft ( void ){
	e_set_speed_left( -turnSpeed );
	e_set_speed_right( turnSpeed );
}
